MenuSceneManager.goat_body_anim = {
	"eat",
	"sleeping",
	"idle_var0",
	"idle_var1",
	"idle_var2",
	"idle_korv",
	"sprint",
	"walk_fwd",
	"back",
	--"jump",
	"left_strafe",
	"right_strafe",
	"attack_feet",
	"attack_head",
	"drink_fountain",
	"hang_fan",
	"hang_lamp",
	"hanging_scaffold",
}
MenuSceneManager.goat_head_anim = {
	"yell_baah",
	"head_hurt_1",
	"head_hurt_2",
	"hit_head_up",
	"hit_head_dwn",
	"hit_head_circle",
}
function MenuSceneManager:_setup_sound_listener()
	self._listener_id = managers.listener:add_listener("player_camera", self._camera_object, self._camera_object, nil, false)
	managers.listener:add_set("player_camera", {
		"player_camera"
	})
	self._listener_activation_id = managers.listener:activate_set("main", "player_camera")
	self._sound_check_object = managers.sound_environment:add_check_object({
		object = self._camera_object,
		active = true,
		primary = true
	})
end
function MenuSceneManager:_clear_sound_listener()
	managers.sound_environment:remove_check_object(self._sound_check_object)
	managers.listener:remove_listener(self._listener_id)
	managers.listener:remove_set("player_camera")
	self._listener_id = nil
end
local init = MenuSceneManager.init
function MenuSceneManager:init()
	init(self)
end
local update = MenuSceneManager.update
function MenuSceneManager:update(t, dt)
	update(self, t, dt)
	local goat = (self._lobby_goat and self._lobby_goat:enabled() and self._lobby_goat) or self._goat_unit
	if not alive(goat) then
		return
	end
	if not self._goat_body_anim_t or self._goat_body_anim_t < t then
		self:goat_play_anim(goat, self.goat_body_anim[math.random(#self.goat_body_anim)])
		self._goat_body_anim_t  = t + 30
	end
	if not self._goat_head_anim_t or self._goat_head_anim_t < t then
		self:goat_play_anim(goat, self.goat_head_anim[math.random(#self.goat_head_anim)])
		goat:sound_source(Idstring("snd")):post_event("goat_says_meh")
		self._goat_head_anim_t  = t + math.random(3,8)
	end
end
local setup_camera = MenuSceneManager.setup_camera
function MenuSceneManager:setup_camera()
	setup_camera(self)
	self:_setup_sound_listener()
end


local _setup_bg = MenuSceneManager._setup_bg
function MenuSceneManager:_setup_bg()
	_setup_bg(self)
	self._menu_logo:set_enabled(false)
	if self._ws then
		World:gui():destroy_workspace(self._ws)
	end
	local rot = Rotation(0, 0, 0)
	local w,h = 310, 70
	self._ws = World:gui():create_world_workspace(w, h, Vector3(-50,233, -55), rot:x() * w, -rot:z() * h)
	self._ws:panel():text({
		text = "Goat Simulator",
		font = tweak_data.menu.pd2_large_font,
		align = "center",
		vertical = "center",
		font_size = 65,
		color = Color.white,
		layer = 2
	})
	--goat
	self._lobby_goat = self:spawn_goat()
	self._lobby_goat:set_enabled(false)
end
local set_scene_template = MenuSceneManager.set_scene_template
function MenuSceneManager:set_scene_template(template, data, custom_name, skip_transition)
	if not skip_transition and (self._current_scene_template == template or self._current_scene_template == custom_name) then
		return
	end
	if not skip_transition and self._ws then
		local template_data = data or self._scene_templates[template]
		if template_data.hide_menu_logo then
			self._ws:hide()
		else
			self._ws:show()
		end
	end
	set_scene_template(self, template, data, custom_name, skip_transition)
end
function MenuSceneManager:setup_goat(goat, parent)
	if alive(parent) then
		goat:unlink()
		parent:link(parent:orientation_object():name(), goat, goat:orientation_object():name())
		goat:set_position(parent:position() + Vector3(0,0,100))
	end
end

function MenuSceneManager:spawn_goat(pos, rot, parent)
	local goat = World:spawn_unit(Idstring("units/pd2_dlc_peta/characters/wld_goat_1/wld_goat_1"), pos or Vector3(), rot or Rotation())
	for i,mat in pairs(goat:get_objects_by_type(Idstring("material")) or {})do
		mat:set_variable(Idstring("contour_opacity"), 0)
	end
	return goat
end
function MenuSceneManager:goat_play_anim(goat, name)
	if not alive(goat) then
		return
	end
	--goat:sound_source():stop()
	goat:play_redirect(Idstring(name))
end

function MenuSceneManager:get_local_peer_id()
	if managers.network and managers.network:session() then
		return managers.network:session():local_peer():id()
	end
end

local _set_character_unit = MenuSceneManager._set_character_unit
function MenuSceneManager:_set_character_unit(...)
	local unit = _set_character_unit(self, ...)
	if not alive(self._goat_unit) then
		self._goat_unit = self:spawn_goat(nil,nil,unit)
	end
	self:setup_goat(self._goat_unit, unit)
	return unit
end
local _set_player_character_unit = MenuSceneManager._set_player_character_unit
function MenuSceneManager:_set_player_character_unit(unit_name)
	_set_player_character_unit(self, unit_name)
	self._character_visibilities[self._character_unit:key()] = false
end
local set_lobby_character_visible = MenuSceneManager.set_lobby_character_visible
function MenuSceneManager:set_lobby_character_visible(i, visible, no_state)
	local id = self:get_local_peer_id()
	if i == id then
		local parent = self._lobby_characters[i]
		self:setup_goat(self._lobby_goat, parent)
		self._lobby_goat:set_enabled(visible)
		self._lobby_goat:set_visible(visible)
		visible = false
	end
	set_lobby_character_visible(self, i, visible, no_state)
end
local _set_unit_enabled_tree = MenuSceneManager._set_unit_enabled_tree
function MenuSceneManager:_set_unit_enabled_tree(unit, state)
	if unit == self._goat_unit or unit == self._lobby_goat then
		return
	end
	_set_unit_enabled_tree(self, unit, state)
	if (alive(self._goat_unit) and unit == self._goat_unit:parent()) or (alive(self._lobby_goat) and unit == self._lobby_goat:parent()) then
		unit:set_enabled(true)
	end
	unit:set_visible(state)
end